#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Optional, Dict, Any

from oser.core import Enum as _Enum
from oser.types import OserElementType, OserStructType, FuzzingValues


class Enum(_Enum):
    """
    An enumeration serializer.
    """

    def __init__(
        self,
        prototype: OserElementType,
        values: Dict[OserElementType, Any],
        value: Optional[Any] = None,
        strict: bool = False,
    ) -> None:
        """
        Args:
            prototype: the class of the underlying data type.
            values: a dictionary of key-value-pairs. The keys are translated into binary data represented by the
                values.
            value=None (object): the initial value.
            strict=False (bool): If set to `True` only keys and values appearing in `values` are accepted. Else all
                values are accepted.
        """
        super(Enum, self).__init__(prototype=prototype, values=values, value=value, strict=strict)

    def get_byte_size(self) -> int:  # type: ignore
        """
        Return the size in bytes.
        """
        return super(Enum, self).get_byte_size()

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded. The user normally does not need to
                supply this.
            context_data (bytes): the binary data of the current context. The user normally does not need to supply
                this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(Enum, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(Enum, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> Any:
        """
        Return the value as string representation
        if possible of the raw value else.

        Returns:
            object: the value as string or raw
        """
        return super(Enum, self).get()

    def get_value(self) -> Any:
        """
        Return the raw value.

        Returns:
            object: the raw value.
        """
        return super(Enum, self).get_value()

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(Enum, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(Enum, self).root()

    def set(self, value: Any) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(Enum, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(Enum, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the size in bits or bytes, depending on the prototype.
        """
        return super(Enum, self).get_size()

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(Enum, self).up()
