#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Optional, Any

from oser.core import Null as _Null
from oser.types import OserElementType, OserStructType, FuzzingValues


class Null(_Null):
    """
    A null-object that does not contain any data.
    """

    def __init__(self) -> None:
        """
        Args:
            value: the initial value.
            format=None (string): Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary.
        """
        super(Null, self).__init__()

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded (always 0).

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(Null, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string. This is always an empty string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(Null, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> str:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(Null, self).get()

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(Null, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(Null, self).root()

    def set(self, value: Any) -> None:
        """
        Set the value. Always raises an Exception.

        Args:
            value: the new value
        """
        return super(Null, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values (iterable): the values used for fuzzing.
        """
        return super(Null, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(Null, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(Null, self).up()
