#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Union, Optional

from oser.core import Padding as _Padding
from oser.types import OserElementType, OserStructType, FuzzingValues


class Padding(_Padding):
    """
    Unsigned 8-Bit integer to fill data with a pattern.
    """

    def __init__(
        self, value: Union[bytes, str, int] = b"\x00", strict: bool = False, format: Optional[str] = None
    ) -> None:
        """
        Args:
            value=b"\x00": the value to be used as the pattern.
            strict: if set to True input data is compared to `value`. In case of a mismatch a `ValueError` is raised.
            format=None: Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary.
        """
        super(Padding, self).__init__(value=value, strict=strict, format=format)

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(Padding, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be encoded.
                The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.
        """
        return super(Padding, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(Padding, self).get()

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(Padding, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(Padding, self).root()

    def set(self, value: Union[bytes, str, int]) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(Padding, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values: the values used for fuzzing.
        """
        return super(Padding, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(Padding, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(Padding, self).up()
