#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Any
from typing import Union, Callable, Optional, Dict, Tuple, List, Generator

from oser.core import RepeatUntil as _RepeatUntil
from oser.types import OserElementType, OserStructType


class RepeatUntil(_RepeatUntil):
    """
    ``RepeatUntil`` builds an array serializer with a variable length that is determined dynamically on decoding.
    """

    def __init__(
        self,
        length: int,
        stop_condition: Optional[Union[Callable, bool]],
        prototype: OserElementType,
        stop_on_exception: bool = False,
        args: Optional[Tuple] = None,
        kwargs: Optional[Dict] = None,
        values: Optional[List[OserElementType]] = None,
    ) -> None:
        """
        Args:
            length: states the initial length
            stop_condition: the stop condition called with ``context``, ``data``
                (the next bytes to be parsed), ``full_data`` (all data) and ``context_data``
                the data within the current hierarchy. If ``stop_condition`` returns ``True`` decoding stops.
            prototype: a class that is the prototype for `values`
            stop_on_exception: if set to ``True`` a ``DecodeException`` is handled as a stop condition.
            args=None: a tuple of positional arguments that are passed to `prototype` when it is instantiated
                when array size is extended.
                Note: don't forget a leading comma when passing one value ``(1,)`` (``(1)`` is an integer!).
            kwargs=None: a dictionary of named arguments that are passed to `prototype` when it is instantiated
                when array size is extended.
            values=None: the initial array values. Can be left empty.
                If empty the array is automatically resized with instances of ``prototype(*args, **kwargs)``.
        """
        super(RepeatUntil, self).__init__(
            length=length,
            stop_condition=stop_condition,
            prototype=prototype,
            stop_on_exception=stop_on_exception,
            args=args,
            kwargs=kwargs,
            values=values,
        )

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be decoded. The user normally does not need to
                supply this.
            context_data: the binary data of the current context. The user normally does not need to supply
                this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(RepeatUntil, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(RepeatUntil, self).encode(full_data=full_data, context_data=context_data)

    def from_dict(self, data: Dict) -> None:
        """
        Fill self with data.

        Args:
            data: data to be used to fill the calling instance.
        """
        return super(RepeatUntil, self).from_dict(data=data)

    def fuzzing_iterator(self, copy: bool = False) -> Generator[Any, None, None]:
        """
        The fuzzing iterator iterates over all combinations
        of set fuzzing values (:func:`oser.ByteType.set_fuzzing_values`).
        If no fuzzing values are set the current struct is yielded.

        Args:
            copy=False: if set to ``True`` the generated
                fuzzing values are deep copies of the original.
                Creating these deep copies is slow. If set to ``False`` the
                original struct is retruned and the generated value must be
                used immediately since the next generated overwrites the values
                on the same value.

        Yields:
            the fuzzing combinations.
        """
        return super(RepeatUntil, self).fuzzing_iterator(copy=copy)

    def get(self) -> "RepeatUntil":
        """
        Return the value.
        """
        return super(RepeatUntil, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(RepeatUntil, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(RepeatUntil, self).root()

    def set_length(self, length: int) -> None:
        """
        Set the length.

        Args:
            length: the new length
        """
        return super(RepeatUntil, self).set_length(length=length)

    def get_size(self) -> int:
        """
        Return the current size in bytes.
        """
        return super(RepeatUntil, self).get_size()

    get_byte_size = get_size

    def to_dict(self) -> List[Dict[Any, Any]]:  # type: ignore
        """
        Return self as a dict()
        """
        return super(RepeatUntil, self).to_dict()

    def up(self) -> OserStructType:
        """
        return parent element
        """
        return super(RepeatUntil, self).up()
