#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Optional

from oser.core import SBVarInt as _SBVarInt
from oser.core import SLVarInt as _SLVarInt
from oser.core import UBVarInt as _UBVarInt
from oser.core import ULVarInt as _ULVarInt
from oser.types import FuzzingValues
from oser.types import OserElementType, OserStructType


class SBVarInt(_SBVarInt):
    """
    Signed, big endian variable length integer.
    """

    def __init__(self, value: int = 0, format: Optional[str] = None) -> None:
        """
        Args:
            value=0: the initial value.
            format=None: Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary.
        """
        super(SBVarInt, self).__init__(value=value, format=format)

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be
                decoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(SBVarInt, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(SBVarInt, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(SBVarInt, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(SBVarInt, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(SBVarInt, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        super(SBVarInt, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values: the values used for fuzzing.
        """
        return super(SBVarInt, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(SBVarInt, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(SBVarInt, self).up()


class SLVarInt(_SLVarInt):
    """
    Signed, little endian variable length integer.
    """

    def __init__(self, value: int = 0, format: Optional[str] = None) -> None:
        """
        Args:
            value=0: the initial value.
            format=None: Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary.
        """
        super(SLVarInt, self).__init__(value=value, format=format)

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be
                decoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(SLVarInt, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(SLVarInt, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(SLVarInt, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(SLVarInt, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(SLVarInt, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        super(SLVarInt, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values: the values used for fuzzing.
        """
        return super(SLVarInt, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(SLVarInt, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(SLVarInt, self).up()


class UBVarInt(_UBVarInt):
    """
    Unsigned, big endian variable length integer.
    """

    def __init__(self, value: int = 0, format: Optional[str] = None) -> None:
        """
        Args:
            value=0: the initial value.
            format=None: Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary.
        """
        super(UBVarInt, self).__init__(value=value, format=format)

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be
                decoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(UBVarInt, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(UBVarInt, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> None:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(UBVarInt, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(UBVarInt, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(UBVarInt, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        super(UBVarInt, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values: the values used for fuzzing.
        """
        return super(UBVarInt, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(UBVarInt, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(UBVarInt, self).up()


class ULVarInt(_ULVarInt):
    """
    Unsigned, little endian variable length integer.
    """

    def __init__(self, value: int = 0, format: Optional[str] = None) -> None:
        """
        Args:
            value=0: the initial value.
            format=None: Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary.
        """
        super(ULVarInt, self).__init__(value=value, format=format)

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be
                decoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(ULVarInt, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(ULVarInt, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> int:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(ULVarInt, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(ULVarInt, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(ULVarInt, self).root()

    def set(self, value: int) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        super(ULVarInt, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values: the values used for fuzzing.
        """
        return super(ULVarInt, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(ULVarInt, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(ULVarInt, self).up()
